/**
 * This is source code for sView.
 *
 * Copyright © Kirill Gavrilov, 2011
 */

#include <StGLWidgets/StGLFont.h>

#include <StFile/StFileNode.h>

void StGLFont::release() {
    if(myFTFace != NULL) {
        FT_Done_Face(myFTFace);
        myFTFace = NULL;
    }
}

bool StGLFont::loadGlyph(const stUtf32_t theChar) {
    myGlyphImg.nullify();
    return theChar != 0
        && FT_Load_Char(myFTFace, theChar, FT_LOAD_TARGET_NORMAL) == 0
        && myFTFace->glyph != NULL;
}

bool StGLFont::loadAndRenderGlyph(const stUtf32_t theChar) {
    myGlyphImg.nullify();
    if(theChar == 0
    || FT_Load_Char(myFTFace, theChar, FT_LOAD_RENDER | FT_LOAD_TARGET_NORMAL) != 0
    || myFTFace->glyph == NULL
    || myFTFace->glyph->format != FT_GLYPH_FORMAT_BITMAP) {
        return false;
    }

    FT_Bitmap aBitmap = myFTFace->glyph->bitmap;
    if(aBitmap.pixel_mode != FT_PIXEL_MODE_GRAY
    || aBitmap.buffer == NULL || aBitmap.width <= 0 || aBitmap.rows <= 0) {
        return false;
    }
    if(!myGlyphImg.initWrapper(StImagePlane::ImgGray, aBitmap.buffer,
                               aBitmap.width, aBitmap.rows, std::abs(aBitmap.pitch))) {
        return false;
    }
    myGlyphImg.setTopDown(aBitmap.pitch > 0);
    return true;
}

StGLFont::~StGLFont() {
    release();
}

StGLFont::StGLFont(const StString& theFontPath,
                   const unsigned int thePointSize,
                   const unsigned int theResolution)
: StGLTextureAtlas(32, 32, 512, 512, GL_ALPHA/**const GLsizei theTileSizeX,
                   const GLsizei theTileSizeY,
                   const GLsizei theTextureSizeX,
                   const GLsizei theTextureSizeY, GL_ALPHA*/),
  myFTFace(NULL),
  myFontPath(theFontPath) {
    StString aFontPath = StFileNode::getCompatibleName(myFontPath);
    if(myFTLib.isNull() || !myFTLib->isValid()
    || FT_New_Face(myFTLib->getInstance(), aFontPath.toCString(), 0, &myFTFace) != 0) {
        ST_DEBUG_LOG("Font '" + aFontPath + "' fail to load!");
        myFTFace = NULL;
        return;
    } else if(FT_Select_Charmap(myFTFace, FT_ENCODING_UNICODE) != 0) {
        ST_DEBUG_LOG("Font '" + aFontPath + "' doesn't contains Unicode charmap!");
        release();
        return;
    } else if(FT_Set_Char_Size(myFTFace, 0L, thePointSize * 64, theResolution, theResolution) != 0) {
        ST_DEBUG_LOG("Font '" + aFontPath + "' doesn't contains Unicode charmap!");
        release();
        return;
    }

}

bool StGLFont::renderGlyph(const size_t    theTileId,
                           const stUtf32_t theChar) {
    if(!loadAndRenderGlyph(theChar)) {
        return false;
    }

    StHandle<StGLTexture> aTexture;///StHandle<StGLTexture>& aTexture = getTexture(theTileId);
    StGLTile& aTile = changeTile(theTileId);
    StVec2<GLsizei> aPos = getTilePx(theTileId);
    /**if(!aTexture->fill(myGlyphImg, const GLsizei theRowFrom = 0, const GLsizei theRowTo = 0)) {
        return false;
    }*/

    const FT_Bitmap& aBitmap = myFTFace->glyph->bitmap;

    ///glBindTexture(GL_TEXTURE_2D, myTextureID);
    ///glTexSubImage2D(GL_TEXTURE_2D, 0, aPos.x(), aPos.y(), aBitmap->width, aBitmap->rows, GL_ALPHA, GL_UNSIGNED_BYTE, aBitmap.buffer);

    aTile.px.left()   = GLfloat(myFTFace->glyph->bitmap_left);
    aTile.px.top()    = GLfloat(myFTFace->glyph->bitmap_top);
    aTile.px.right()  = aTile.px.left() + GLfloat(aBitmap.width);
    aTile.px.bottom() = aTile.px.top()  + GLfloat(aBitmap.rows);

    aTile.uv.left()   = (GLfloat(aPos.x()) / GLfloat(aTexture->getSizeX()));
    aTile.uv.top()    = (GLfloat(aPos.y()) / GLfloat(aTexture->getSizeY()));
    aTile.uv.right()  = (GLfloat(aPos.x() + aBitmap.width)  / GLfloat(aTexture->getSizeX()));
    aTile.uv.bottom() = (GLfloat(aPos.y() + aBitmap.rows)   / GLfloat(aTexture->getSizeY()));
}

GLfloat StGLFont::getAdvance(const stUtf32_t theUChar1,
                             const stUtf32_t theUChar2) {
    if(!loadGlyph(theUChar1)) {
        return 0.0f;
    }
    if(FT_HAS_KERNING(myFTFace) == 0 || theUChar2 == 0
    || FT_Get_Kerning(myFTFace, theUChar1, theUChar2, FT_KERNING_UNFITTED, &myKernAdvance) != 0) {
        return GLfloat(myFTFace->glyph->advance.x) / 64.0f;
    }
    return GLfloat(myKernAdvance.x + myFTFace->glyph->advance.x) / 64.0f;
}

void StGLFont::addAdvance(const stUtf32_t theUChar1,
                          const stUtf32_t theUChar2,
                          StGLVec2&       theVec) {
    if(!loadGlyph(theUChar1)) {
        return;
    }
    if(FT_HAS_KERNING(myFTFace) == 0 || theUChar2 == 0
    || FT_Get_Kerning(myFTFace, theUChar1, theUChar2, FT_KERNING_UNFITTED, &myKernAdvance) != 0) {
        theVec.x() += GLfloat(myFTFace->glyph->advance.x) / 64.0f;
        theVec.y() += GLfloat(myFTFace->glyph->advance.y) / 64.0f;
    } else {
        theVec.x() += GLfloat(myKernAdvance.x + myFTFace->glyph->advance.x) / 64.0f;
        theVec.y() += GLfloat(myKernAdvance.y + myFTFace->glyph->advance.y) / 64.0f;
    }
}
