/**
 * Copyright © 2009-2011 Kirill Gavrilov <kirill@sview.ru>
 *
 * StMoviePlayer program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * StMoviePlayer program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program.
 * If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __StSeekBar_h_
#define __StSeekBar_h_

#include <StGLWidgets/StGLWidget.h>
#include <StGL/StGLVertexBuffer.h>

/**
 * Simple seeking bar class.
 */
class ST_LOCAL StSeekBar : public StGLWidget {

        private:

    StGLVertexBuffer stVerticesBuf;
    StGLVertexBuffer stColorsBuf;

    GLfloat progress;

    class StProgramSB;
    StProgramSB* stProgram;

    int progressPx;

    void stglUpdateVertices();

        public:

    StSeekBar(StGLWidget* parent, const int& top = -64);
    virtual ~StSeekBar();

    virtual const StString& getClassName();

    /**
     * @param progress (GLfloat ) - current progress from 0.0f to 1.0f;
     */
    void setProgress(GLfloat progress) {
        this->progress = progress;
    }

    virtual void stglResize(const StRectI_t& winRectPx);
    virtual bool stglInit();
    virtual void stglDraw(unsigned int view);

        private: //!< callback Slots (private overriders)

    void doMouseUnclick(const int );

        public:  //!< Signals

    struct {
        /**
         * Emit callback Slot on mouse click.
         * @param theMouseBtnId (const int ) - mouse button id;
         * @param theProgress (const double ) - current progress value.
         */
        StSignal<void (const int , const double )> onSeekClick;
    } signals;

};

#endif //__StSeekBar_h_
