/**
 * Copyright © 2009-2011 Kirill Gavrilov <kirill@sview.ru>
 *
 * StMoviePlayer program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * StMoviePlayer program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program.
 * If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __StTimeBox_h_
#define __StTimeBox_h_

#include <StGLWidgets/StGLTextureButton.h>
#include <StGLWidgets/StGLTextArea.h>
#include <StStrings/StFormatTime.h>

class ST_LOCAL StTimeBox : public StGLTextureButton {

        private:

    StGLTextArea* textArea;
    double   progressSec;
    double   durationSec;
    bool       isElapsed;

        public:

    StTimeBox(StGLWidget* parent, const int& left = 32, const int& top = 32)
    : StGLTextureButton(        parent, left, top, StGLCorner(ST_VCORNER_TOP, ST_HCORNER_LEFT)),
      textArea(new StGLTextArea(parent, left, top, StGLCorner(ST_VCORNER_TOP, ST_HCORNER_LEFT), 32, 32)),
      progressSec(0.0),
      durationSec(0.0),
      isElapsed(true) {
        //
        textArea->setBorder(false);
        textArea->setTextColor(1.0f, 1.0f, 1.0f);
        StGLWidget::signals.onMouseUnclick.connect(this, &StTimeBox::doSwitchElapsed);
    }

    virtual ~StTimeBox() {
        //
    }

    virtual const StString& getClassName() {
        static const StString className("StTimeBox");
        return className;
    }

    virtual bool stglInit() {
        bool btnInit = StGLTextureButton::stglInit();
        textArea->changeRectPx() = StGLTextureButton::getRectPx();
        textArea->changeRectPx().top() += 26;
        return btnInit && textArea->stglInit();
    }

    virtual void stglDraw(unsigned int view) {
        if(isElapsed) {
            textArea->setText(StFormatTime::formatSeconds(progressSec) + " / " + StFormatTime::formatSeconds(durationSec));
        } else {
            textArea->setText(StFormatTime::formatSeconds(progressSec - durationSec));
        }
        StGLTextureButton::stglDraw(view);
        textArea->stglDraw(view);
    }

    void setVisibility(bool isVisible) {
        StGLTextureButton::setVisibility(isVisible);
        textArea->setVisibility(isVisible);
    }

    void setTime(const double& progressSec, const double& durationSec) {
        this->progressSec = progressSec;
        this->durationSec = durationSec;
    }

        public: //!< callback Slots

    void doSwitchElapsed(const int ) {
        isElapsed = !isElapsed;
    }

};

#endif //__StTimeBox_h_
