/**
 * Copyright © 2007-2010 Kirill Gavrilov <kirill@sview.ru>
 *
 * StOutAnaglyph library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * StOutAnaglyph library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library.
 * If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __StOutAnaglyph_h_
#define __StOutAnaglyph_h_

#include <StCore/StCore.h>     // header for Stereo Output Core
#include <StGLStereo/StGLStereoFrameBuffer.h>
#include <StThreads/StFPSControl.h>

class StSettings;

class ST_LOCAL StOutAnaglyph : public StRendererInterface {

        private:

    enum {
        DEVICE_AUTO     =-1,
        DEVICE_ANAGLYPH = 0, // Anaglyph glasses
    };

    enum {
        GLASSES_TYPE_REDCYAN = 0, // Red-Cyan glasses (R + GB)
        GLASSES_TYPE_YELLOW  = 1, // Yellow glasses (RG + B)
        GLASSES_TYPE_GREEN   = 2, // Green-Magenta glasses (G + RB)
    };

    enum {
        REDCYAN_MODE_SIMPLE = 0, // simple Red-Cyan anaglyph
        REDCYAN_MODE_OPTIM  = 1, // optimized Red-Cyan anaglyph
        REDCYAN_MODE_GRAY   = 2, // grayed Red-Cyan anaglyph
        REDCYAN_MODE_DARK   = 3, // dark Red-Cyan anaglyph
    };

    enum {
        AMBERBLUE_MODE_SIMPLE = 0, // simple Amber-Blue anaglyph
        AMBERBLUE_MODE_DUBOIS = 1, // optimized Amber-Blue anaglyph
    };

    enum {
        DEVICE_OPTION_VSYNC   = 0,
        DEVICE_OPTION_SHOWFPS = 1,
        DEVICE_OPTION_GLASSES = 2,
        DEVICE_OPTION_REDCYAN = 3,
        DEVICE_OPTION_YELLOW  = 4,
    };

    StCore*            stCore;
    StSettings*    stSettings;
    StString    pluginPath;

    StGLStereoFrameBuffer* frBuffer;
    StGLStereoFrameBuffer::StGLStereoProgram*   stereoProgram; // pointer to current anaglyph program

    StGLStereoFrameBuffer::StGLStereoProgram stSimpleAnaglyph; // all available anaglyph programs
    StGLStereoFrameBuffer::StGLStereoProgram   stGrayAnaglyph;
    StGLStereoFrameBuffer::StGLStereoProgram   stTrueAnaglyph;
    StGLStereoFrameBuffer::StGLStereoProgram  stOptimAnaglyph;
    StGLStereoFrameBuffer::StGLStereoProgram stYellowAnaglyph;
    StGLStereoFrameBuffer::StGLStereoProgram stYellowDubiosAnaglyph;
    StGLStereoFrameBuffer::StGLStereoProgram  stGreenAnaglyph;
    int               glasses;
    int         optionRedCyan;
    int       optionAmberBlue;

    StSDOptionsList_t* optionsStruct;

    StFPSControl stFPSControl;
    bool      toSavePlacement;
    bool              isVSync;
    bool        isUserFrmSize; // user-specified frame buffer size
    bool            isShowFPS;

    void setShader(const int theGlasses,
                   const int theOptionRedCyan,
                   const int theOptionAmberBlue);
    void optionsStructAlloc();

        public:

    StOutAnaglyph();
    ~StOutAnaglyph();
    StRendererInterface* getLibImpl() { return this; }
    StWindowInterface* getStWindow() { return stCore->getStWindow(); }
    bool init(const StString& , const int& , const StNativeWin_t* );
    bool open(const StOpenInfo& stOpenInfo) { return stCore->open(stOpenInfo); }
    void callback(StMessage_t* );
    void stglDraw(unsigned int );

};

#endif //__StOutAnaglyph_h_
