/**
 * Copyright © 2007-2010 Kirill Gavrilov <kirill@sview.ru>
 *
 * StOutDual library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * StOutDual library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library.
 * If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __StOutDual_h_
#define __StOutDual_h_

#include <StCore/StCore.h> // header for Stereo Output Core
#include <StGL/StGLVertexBuffer.h>
#include <StThreads/StFPSControl.h>

class StSettings;
class StProgramMM;
class StGLFrameBuffer;

class ST_LOCAL StOutDual : public StRendererInterface {

        private:

    enum {
        DUALMODE_SIMPLE   = 0, // no mirroring
        DUALMODE_XMIRROW  = 1, // mirror on X SLAVE  window
        DUALMODE_YMIRROW  = 2, // mirror on Y SLAVE  window
    };

    enum {
        DEVICE_OPTION_VSYNC = 0,
        DEVICE_OPTION_SHOWFPS = 1,
    };

    StCore*          stCore;
    StSettings*  stSettings;
    StString  pluginPath;

    StGLFrameBuffer*      frBuffer; // OpenGL frame buffer object
    StProgramMM*         stProgram;
    StFPSControl      stFPSControl;
    StGLVertexBuffer stVertFlatBuf; // buffers to draw simple fullsreen quad
    StGLVertexBuffer stVertXMirBuf;
    StGLVertexBuffer stVertYMirBuf;
    StGLVertexBuffer stTexCoordBuf;

    StSDOptionsList_t* optionsStruct;

    int            deviceId;
    int32_t      slaveMovId;
    bool    toSavePlacement;
    bool            isVSync;
    bool      isUserFrmSize; // user-specified frame buffer size
    bool          isShowFPS;

    void replaceDualAttribute(const int& from, const int& to);
    void optionsStructAlloc();

        public:

    StOutDual();
    ~StOutDual();
    StRendererInterface* getLibImpl() { return this; }
    StWindowInterface* getStWindow() { return stCore->getStWindow(); }
    bool init(const StString& , const int& , const StNativeWin_t* );
    bool open(const StOpenInfo& stOpenInfo) { return stCore->open(stOpenInfo); }
    void callback(StMessage_t* );
    void stglDraw(unsigned int );

};

#endif //__StOutDual_h_
