/**
 * Copyright © 2009 Kirill Gavrilov <kirill@sview.ru>
 *
 * StOutIZ3D library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * StOutIZ3D library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library.
 * If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __StOutIZ3D_h_
#define __StOutIZ3D_h_

#include <StCore/StCore.h>       // header for Stereo Output Core
#include <StThreads/StThreads.h> // threads header (mutexes, threads,...)
#include <StThreads/StFPSControl.h>
#include <StGL/StGL.h>
#include <StGL/StGLTexture.h>

#include "StOutIZ3DShaders.h"

class StSettings;

class ST_LOCAL StOutIZ3D : public StRendererInterface {

        private:

    enum {
        DEVICE_OPTION_VSYNC = 0,
        DEVICE_OPTION_SHOWFPS = 1,
        DEVICE_OPTION_SHADER = 2,
    };

    StCore*            stCore;
    StSettings*    stSettings;
    StString    pluginPath;

    StGLStereoFrameBuffer* frBuffer; // frame buffer to draw
    StOutIZ3DShaders   shIZ3D; // IZ3D shaders
    StGLTexture stTexTableOld; // table textures
    StGLTexture stTexTableNew;

    StSDOptionsList_t* optionsStruct;

    StFPSControl stFPSControl;
    bool      toSavePlacement;
    bool              isVSync;
    bool        isUserFrmSize; // user-specified frame buffer size
    bool            isShowFPS;

    void optionsStructAlloc();

        public:

    StOutIZ3D();
    ~StOutIZ3D();
    StRendererInterface* getLibImpl() { return this; }
    StWindowInterface* getStWindow() { return stCore->getStWindow(); }
    bool init(const StString& , const int& , const StNativeWin_t* );
    bool open(const StOpenInfo& stOpenInfo) { return stCore->open(stOpenInfo); }
    void callback(StMessage_t* );
    void stglDraw(unsigned int );

};

#endif //__StOutIZ3D_h_
