/**
 * Copyright © 2007-2010 Kirill Gavrilov <kirill@sview.ru>
 *
 * Distributed under the Boost Software License, Version 1.0.
 * See accompanying file license-boost.txt or copy at
 * http://www.boost.org/LICENSE_1_0.txt
 */

#include <StCore/StWindow.h>
#include <StLibrary.h>

StWindow::WindowFunctions::WindowFunctions()
: StWindow_new(NULL),
  StWindow_del(NULL),
  StWindow_close(NULL),
  StWindow_setTitle(NULL),
  StWindow_getAttributes(NULL),
  StWindow_setAttributes(NULL),
  StWindow_isStereoOutput(NULL),
  StWindow_setStereoOutput(NULL),
  StWindow_show(NULL),
  StWindow_showCursor(NULL),
  StWindow_isFullScreen(NULL),
  StWindow_setFullScreen(NULL),
  StWindow_getPlacement(NULL),
  StWindow_setPlacement(NULL),
  StWindow_getMousePos(NULL),
  StWindow_getMouseDown(NULL),
  StWindow_getMouseUp(NULL),
  StWindow_getDragNDropFile(NULL),
  StWindow_stglCreate(NULL),
  StWindow_stglSwap(NULL),
  StWindow_stglMakeCurrent(NULL),
  StWindow_stglGetTargetFps(NULL),
  StWindow_stglSetTargetFps(NULL),
  StWindow_callback(NULL),
  StWindow_appendMessage(NULL),
  StWindow_getValue(NULL),
  StWindow_setValue(NULL),
  StWindow_memAlloc(NULL),
  StWindow_memFree(NULL) {
    //
}

StWindow::WindowFunctions::~WindowFunctions() {}

void StWindow::WindowFunctions::load(StLibrary& theLib) {
    theLib("StWindow_new",              StWindow_new);
    theLib("StWindow_del",              StWindow_del);
    theLib("StWindow_close",            StWindow_close);
    theLib("StWindow_setTitle",         StWindow_setTitle);
    theLib("StWindow_getAttributes",    StWindow_getAttributes);
    theLib("StWindow_setAttributes",    StWindow_setAttributes);
    theLib("StWindow_isStereoOutput",   StWindow_isStereoOutput);
    theLib("StWindow_setStereoOutput",  StWindow_setStereoOutput);
    theLib("StWindow_show",             StWindow_show);
    theLib("StWindow_showCursor",       StWindow_showCursor);
    theLib("StWindow_isFullScreen",     StWindow_isFullScreen);
    theLib("StWindow_setFullScreen",    StWindow_setFullScreen);
    theLib("StWindow_getPlacement",     StWindow_getPlacement);
    theLib("StWindow_setPlacement",     StWindow_setPlacement);
    theLib("StWindow_getMousePos",      StWindow_getMousePos);
    theLib("StWindow_getMouseDown",     StWindow_getMouseDown);
    theLib("StWindow_getMouseUp",       StWindow_getMouseUp);
    theLib("StWindow_getDragNDropFile", StWindow_getDragNDropFile);
    theLib("StWindow_stglCreate",       StWindow_stglCreate);
    theLib("StWindow_stglSwap",         StWindow_stglSwap);
    theLib("StWindow_stglMakeCurrent",  StWindow_stglMakeCurrent);
    theLib("StWindow_stglGetTargetFps", StWindow_stglGetTargetFps);
    theLib("StWindow_stglSetTargetFps", StWindow_stglSetTargetFps);
    theLib("StWindow_callback",         StWindow_callback);
    theLib("StWindow_appendMessage",    StWindow_appendMessage);
    theLib("StWindow_getValue",         StWindow_getValue);
    theLib("StWindow_setValue",         StWindow_setValue);
    theLib("StWindow_memAlloc",         StWindow_memAlloc);
    theLib("StWindow_memFree",          StWindow_memFree);
}

bool StWindow::WindowFunctions::isNull() const {
    return StWindow_new  == NULL || StWindow_del == NULL || StWindow_close == NULL
        || StWindow_callback == NULL
        || StWindow_appendMessage == NULL
        || StWindow_stglSwap == NULL         || StWindow_stglCreate == NULL || StWindow_stglMakeCurrent == NULL
        || StWindow_stglGetTargetFps == NULL || StWindow_stglSetTargetFps == NULL
        || StWindow_setTitle == NULL
        || StWindow_getAttributes == NULL    || StWindow_setAttributes == NULL
        || StWindow_isStereoOutput == NULL   || StWindow_setStereoOutput == NULL
        || StWindow_show == NULL             || StWindow_showCursor == NULL
        || StWindow_isFullScreen == NULL     || StWindow_setFullScreen == NULL
        || StWindow_getPlacement == NULL     || StWindow_setPlacement == NULL
        || StWindow_getMousePos == NULL      || StWindow_getMouseDown == NULL || StWindow_getMouseUp == NULL
        || StWindow_getDragNDropFile == NULL
        || StWindow_getValue == NULL         || StWindow_setValue == NULL
        || StWindow_memAlloc == NULL         || StWindow_memFree == NULL;
}

void StWindow::WindowFunctions::nullify() {
    StWindow_new = NULL;
    StWindow_del = NULL;
    StWindow_close = NULL;
    StWindow_setTitle = NULL;
    StWindow_getAttributes = NULL;
    StWindow_setAttributes = NULL;
    StWindow_isStereoOutput = NULL;
    StWindow_setStereoOutput = NULL;
    StWindow_show = NULL;
    StWindow_showCursor = NULL;
    StWindow_isFullScreen = NULL;
    StWindow_setFullScreen = NULL;
    StWindow_getPlacement = NULL;
    StWindow_setPlacement = NULL;
    StWindow_getMousePos = NULL;
    StWindow_getMouseDown = NULL;
    StWindow_getMouseUp = NULL;
    StWindow_getDragNDropFile = NULL;
    StWindow_stglCreate = NULL;
    StWindow_stglSwap = NULL;
    StWindow_stglMakeCurrent = NULL;
    StWindow_stglGetTargetFps = NULL;
    StWindow_stglSetTargetFps = NULL;
    StWindow_callback = NULL;
    StWindow_appendMessage = NULL;
    StWindow_getValue = NULL;
    StWindow_setValue = NULL;
    StWindow_memAlloc = NULL;
    StWindow_memFree = NULL;
}

namespace {
    static StWindow::WindowFunctions ST_WINDOW_FUNCTIONS;
};

StWindow::WindowFunctions& StWindow::GetFunctions() {
    return ST_WINDOW_FUNCTIONS;
}
