/**
 * This is source code for sView.
 *
 * Copyright © Kirill Gavrilov, 2011
 */

#ifndef __StDataMap_h_
#define __StDataMap_h_

#include <stTypes.h>
#include <map>

template<typename Key_t, typename Value_t>
class StDataMap : public std::map<Key_t, Value_t> {

        public:

    typedef std::map<Key_t, Value_t>        stdMap;
    typedef typename stdMap::iterator       stdMapIter;
    typedef typename stdMap::const_iterator stdMapConstIter;

    StDataMap() : stdMap() {}

    // notice that destructor in std::map doesn't virtual!
    //~StDataMap()

    bool isContains(const Key_t& theKey) {
        stdMapIter anIter = stdMap::find(theKey);
        return anIter != stdMap::end();
    }

    const Value_t& getValue(const Key_t& theKey) const {
        stdMapConstIter anIter = stdMap::find(theKey);
        ST_DEBUG_ASSERT(anIter != stdMap::end());
        return anIter->second;
    }

    bool getValue(const Key_t& theKey,
                  Value_t&     theValue) const {
        stdMapConstIter anIter = stdMap::find(theKey);
        if(anIter != stdMap::end()) {
            theValue = anIter->second;
            return true;
        }
        return false;
    }

    Value_t& changeValue(const Key_t& theKey) {
        stdMapIter anIter = stdMap::find(theKey);
        ST_DEBUG_ASSERT(anIter != stdMap::end()); /// should be exception
        return anIter->second;
    }

    void bindValue(const Key_t&   theKey,
                   const Value_t& theValue) {
        ST_DEBUG_ASSERT(!isContains(theKey)); /// should be exception
        stdMap::operator[](theKey) = theValue;
    }

    void overrideValue(const Key_t&   theKey,
                       const Value_t& theValue) {
        stdMap::operator[](theKey) = theValue;
    }

};

#endif // __StDataMap_h_
