/**
 * This is source code for sView.
 *
 * Copyright © Kirill Gavrilov, 2011
 */

#ifndef __StGLTextureAtlas_h_
#define __StGLTextureAtlas_h_

#include <StGL/StGLVec.h>
#include <StGL/StGLTexture.h>
#include <StTemplates/StRect.h>

typedef StRect<GLfloat> StGLRect;

/**
 * Simple structure stores tile rectangle.
 */
struct StGLTile {

    StGLRect uv; //!< UV coordinates in texture
    StGLRect px; //!< pixel displacement coordinates

        public: //! compare operators

    bool operator==(const StGLTile& theCompare) const {
        return uv == theCompare.uv && px == theCompare.px;
    }

};

template<> inline void StArray< StHandle<StGLTexture> >::sort() {}
template<> inline void StArray<StGLTile>::sort() {}

/**
 * Texture atlas is a texture(s) that contains set of subimages (tiles).
 * This class implements texture atlas of tiles with equal dimensions.
 * Initialization is split into 2 chunks - reserve + init GL resources.
 */
class ST_LOCAL StGLTextureAtlas {

        private:

    GLsizei myTileSizeX;           //!< tile width
    GLsizei myTileSizeY;           //!< tile height
    GLsizei myTextureSizeX;        //!< texture width
    GLsizei myTextureSizeY;        //!< texture height
    size_t  myTilesInTexture;      //!< number of tiles that fits in one texture
    size_t  myLastTile;            //!< id of last tile
    size_t  myLastTileInitialized; //!< id of last tile that was initialized
    GLint   myTextureFormat;       //!< texture format

    StArrayList< StHandle<StGLTexture> > myTextures; //!< texture list
    StArrayList<StGLTile> myTiles;                   //!< tiles list

        public:

    /**
     * Default constructor. No GL object is initialized here.
     * @param theTileSizeX (const GLsizei ) - tile width  in pixels;
     * @param theTileSizeY (const GLsizei ) - tile height in pixels;
     * @param theTextureSizeX (const GLsizei ) - width  of textures in pixels;
     * @param theTextureSizeY (const GLsizei ) - height of textures in pixels;
     * @param theTextureFormat (const GLint ) - texture format.
     */
    StGLTextureAtlas(const GLsizei theTileSizeX,
                     const GLsizei theTileSizeY,
                     const GLsizei theTextureSizeX,
                     const GLsizei theTextureSizeY,
                     const GLint   theTextureFormat);

    /**
     * @param theTileId (const size_t ) - tile id;
     * @return texture id in array.
     */
    size_t getTextureId(const size_t theTileId) const {
        return theTileId / myTilesInTexture;
    }

    /**
     * Access texture for specified tile.
     * @param theTileId (const size_t ) - tile id;
     * @return texture that stores specified tile.
     */
    const StHandle<StGLTexture>& getTexture(const size_t theTileId) const {
        return myTextures[getTextureId(theTileId)];
    }

    /**
     * Access tile by id.
     * @param theTileId (const size_t ) - tile id;
     * @return tile structure.
     */
    const StGLTile& getTile(const size_t theTileId) const {
        return myTiles[theTileId];
    }

    /**
     * Access tile by id.
     * @param theTileId (const size_t ) - tile id;
     * @return tile structure (changeable).
     */
    StGLTile& changeTile(const size_t theTileId) {
        return myTiles[theTileId];
    }

    StVec2<GLsizei> getTilePx(const size_t theTileId) const {
        ///return StVec2<GLsizei>(,);
        ///
    }


    bool reserveMore(const size_t theTilesNb);

    /**
     * @return true if some tiles are in uninitialized state.
     */
    bool hasReserved() const {
        return myLastTile != myLastTileInitialized;
    }

    bool init();

};

#endif // __StGLTextureAtlas_h_
