/**
 * Copyright © 2009-2010 Kirill Gavrilov <kirill@sview.ru>
 *
 * Distributed under the Boost Software License, Version 1.0.
 * See accompanying file license-boost.txt or copy at
 * http://www.boost.org/LICENSE_1_0.txt
 */

#ifndef __StGLDescription_h_
#define __StGLDescription_h_

#include <StGLWidgets/StGLTextArea.h>

class ST_LOCAL StGLDescription : public StGLTextArea {

        public:

    StGLDescription(StGLWidget* parent, const int& width = 256)
    : StGLTextArea(parent, 0, 0, StGLCorner(ST_VCORNER_TOP, ST_HCORNER_LEFT), width, 96) {
        //
        setBorder(true);
    }

    virtual ~StGLDescription() {
        //
    }

    virtual const StString& getClassName() {
        static const StString className("StGLDescription");
        return className;
    }

    /**
     * Make sure you put point in RootWidget!
     * Mouse cursor putted here in most cases, and RootWidget must be window rectangle in this case.
     * @param pointZo (const StPointD_t& ) - point in RootWidget;
     */
    void setPoint(const StPointD_t& pointZo) {
        const StRectI_t& rootRectPx = StGLWidget::getParent()->getRectPx();
        int w = StGLWidget::getRectPx().width();
        int h = StGLWidget::getRectPx().height();

        StRectI_t rect;
        // TODO (Kirill Gavrilov#9#) should we convert pointZo to local coordinates?
        rect.left()   = int(pointZo.x() * (double )rootRectPx.width()) + 16;
        rect.right()  = rect.left() + w;
        rect.top()    = int(pointZo.y() * (double )rootRectPx.height()) + 16;
        rect.bottom() = rect.top() + h;
        int dirH = StGLTextArea::TO_RIGHT;
        int dirV = StGLTextArea::TO_DOWN;
        if(rect.right() > rootRectPx.width()) {
            dirH = StGLTextArea::TO_LEFT;
            rect.left()  -= 32;
            rect.right() -= 32;
        }
        if(rect.bottom() > rootRectPx.height()) {
            dirV = StGLTextArea::TO_UP;
            rect.top()    -= 32;
            rect.bottom() -= 32;
        }
        StGLTextArea::setAreaDirection(dirH | dirV);
        StGLWidget::changeRectPx() = rect;
    }

};

#endif //__StGLDescription_h_
