/**
 * This is source code for sView.
 *
 * Copyright © Kirill Gavrilov, 2011
 */

#ifndef __StGLFont_h_
#define __StGLFont_h_

#include <StGL/StGLVec.h>
#include <StGL/StGLTextureAtlas.h>
#include <StImage/StImagePlane.h>
#include <StCollections/StDataMap.h>

// idiotic inclusion template for FreeType
#include <ft2build.h>
#include FT_FREETYPE_H

class ST_LOCAL StFTLibrary {

        private:

    FT_Library myFTLib;

        public:

    bool isValid() const {
        return myFTLib != NULL;
    }

    StFTLibrary()
    : myFTLib(NULL) {
        if(FT_Init_FreeType(&myFTLib) != 0) {
            myFTLib = NULL;
        }
    }

    ~StFTLibrary() {
        if(isValid()) {
            FT_Done_FreeType(myFTLib);
        }
    }

    FT_Library getInstance() const {
        return myFTLib;
    }

};

class ST_LOCAL StGLFont : public StGLTextureAtlas {

        private:

    StHandle<StFTLibrary> myFTLib;    //!< handle to the FT library object
    FT_Face               myFTFace;   //!< FT face object
    StString              myFontPath; //!< font path
    StImagePlane          myGlyphImg; //!< cached glyph plane

    FT_Vector             myKernAdvance; //!< buffer variable
    StDataMap<stUtf32_t, size_t> myGlyphMap;

        private:

    void release();

    bool loadGlyph(const stUtf32_t theChar);

    bool loadAndRenderGlyph(const stUtf32_t theChar);

    /**StGLVec2 getAdvance() const {
        return StGLVec2(myFTFace->glyph->advance.x / 64.0f,
                        myFTFace->glyph->advance.y / 64.0f);


        bool reserveMore(const size_t theTilesNb = 1)
        StGLTile& aNewTile = changeTile(const size_t theTileId);

    }*/

        public:

    bool isValid() const {
        return myFTFace != NULL;
    }

    ~StGLFont();

    size_t getTileId(const stUtf32_t theUCharCode) const {
        myGlyphMap.getValue(theUCharCode);
    }

    /**
     * @param thePointSize  - the face size in points (1/72 inch);
     * @param theResolution - the resolution of the target device;
     */
    StGLFont(const StString& theFontPath,
             const unsigned int thePointSize,
             const unsigned int theResolution = 72);

    bool renderGlyph(const size_t    theTileId,
                     const stUtf32_t theChar);

    GLfloat getAdvance(const stUtf32_t theUChar1,
                       const stUtf32_t theUChar2);

    void addAdvance(const stUtf32_t theUChar1,
                    const stUtf32_t theUChar2,
                    StGLVec2&       theVec);

};

#endif // __StGLFont_h_
