/**
 * Copyright © 2009-2011 Kirill Gavrilov <kirill@sview.ru>
 *
 * Distributed under the Boost Software License, Version 1.0.
 * See accompanying file license-boost.txt or copy at
 * http://www.boost.org/LICENSE_1_0.txt
 */

#ifndef __StGLMenu_h_
#define __StGLMenu_h_

#include <StGLWidgets/StGLTextArea.h>
#include <StGL/StGLVertexBuffer.h>

// forward declarations
class StBoolParam;
class StInt32Param;
class StFloat32Param;
class StGLMenuItem;
class StGLMenuProgram;

/**
 * Widget represents classical menu object.
 */
class ST_LOCAL StGLMenu : public StGLWidget {

        private:

    StGLShare<StGLMenuProgram> myProgram;
    StGLVertexBuffer stVertexBuf;
    StGLVec4 stColorVec;

    int         orient;
    int     itemHeight;
    int          width;

    bool     bRootMenu;
    bool       bActive;
    bool isInitialized;

    void stglResize();

        public:

    enum {
        MENU_VERTICAL = 0,
        MENU_HORIZONTAL = 1,
    };

    // recursively delete all submenus and that this menu itself
    // should be used only for dynamic menu recreation
    static void DeleteWithSubMenus(StGLMenu* stMenu);

    StGLMenu(StGLWidget* theParent,
             const int theLeft, const int theTop,
             const int theOrient = MENU_VERTICAL,
             const bool theIsRootMenu = false);

    virtual const StString& getClassName();
    virtual void setVisibility(bool isVisible, bool isForce);
    virtual void stglResize(const StRectI_t& winRectPx);
    virtual bool stglInit();
    virtual void stglDraw(unsigned int view);
    virtual bool tryUnClick(const StPointD_t& cursorZo, const int& mouseBtn, bool& isItemUnclicked);

    bool isRootMenu() const {
        return bRootMenu;
    }

    bool isActive() const {
        return bActive;
    }

    void setActive(const bool isActive) {
        this->bActive = isActive;
    }

    int getOrient() const {
        return orient;
    }

    /**
     * Update all children menus layout.
     */
    void stglUpdateSubmenuLayout();

    StGLMenuItem* addItem(const StString& label, const size_t userData = 0);
    StGLMenuItem* addItem(const StString& label, StGLMenu* subMenu);

    /**
     * Append checkbox menu item.
     * @param theLabel (const StString& ) - menu item text;
     * @param theTrackedValue (const StHandle<StBoolParam>& ) - tracked boolean value;
     * @return created menu item widget.
     */
    StGLMenuItem* addItem(const StString& theLabel,
                          const StHandle<StBoolParam>& theTrackedValue);

    /**
     * Append radio button menu item.
     * @param theLabel (const StString& ) - menu item text;
     * @param theTrackedValue (const StHandle<StInt32Param>& ) - tracked integer (enumeration) value;
     * @param theOnValue (const int32_t ) - associated integer (enumeration) value for this radio button;
     * @return created menu item widget.
     */
    StGLMenuItem* addItem(const StString& theLabel,
                          const StHandle<StInt32Param>& theTrackedValue,
                          const int32_t theOnValue);

    /**
     * Append radio button menu item.
     * @param theLabel (const StString& ) - menu item text;
     * @param theTrackedValue (const StHandle<StFloat32Param>& ) - tracked float value;
     * @param theOnValue (const float ) - associated float value for this radio button;
     * @return created menu item widget.
     */
    StGLMenuItem* addItem(const StString& theLabel,
                          const StHandle<StFloat32Param>& theTrackedValue,
                          const float theOnValue);

};

#endif //__StGLMenu_h_
