/**
 * Copyright © 2009-2011 Kirill Gavrilov <kirill@sview.ru>
 *
 * Distributed under the Boost Software License, Version 1.0.
 * See accompanying file license-boost.txt or copy at
 * http://www.boost.org/LICENSE_1_0.txt
 */

#ifndef __StGLMenuItem_h_
#define __StGLMenuItem_h_

#include <StGLWidgets/StGLShare.h>
#include <StGLWidgets/StGLTextArea.h>
#include <StGL/StGLVertexBuffer.h>

class StGLMenu;
class StGLMenuProgram;
class ST_LOCAL StGLMenuItem : public StGLTextArea {

        private:

    typedef enum tagState {
        PASSIVE,
        HIGHLIGHT,
        CLICKED,
    } State;

    StGLMenu* subMenu;

    StGLShare<StGLMenuProgram> myProgram;

    StGLVertexBuffer stBackVertexBuf;       // background
    StGLVec4 stBackColor[3];

    bool   bItemSelected; // navigation selection flag

    void stglResize();

        public:

    // recursively delete all submenus and that this item itself
    // should be used only for dynamic menu recreation
    static void DeleteWithSubMenus(StGLMenuItem* stMenuItem);

    StGLMenuItem(StGLMenu* theParent,
                 const int theLeft = 32, const int theTop = 32,
                 StGLMenu* theSubMenu = NULL);

    virtual const StString& getClassName();
    virtual void stglUpdate(const StPointD_t& cursorZo);
    virtual void stglResize(const StRectI_t& winRectPx);
    virtual bool stglInit();
    virtual void stglDraw(unsigned int view);
    virtual bool tryClick(const StPointD_t& cursorZo, const int& mouseBtn, bool& isItemClicked);
    virtual bool tryUnClick(const StPointD_t& cursorZo, const int& mouseBtn, bool& isItemUnclicked);

    const int computeTextWidth();

    StGLMenu* getParentMenu() {
        return (StGLMenu* )StGLWidget::getParent();
    }

    StGLMenu* getSubMenu() {
        return subMenu;
    }

    void setSubMenu(StGLMenu* subMenu) {
        this->subMenu = subMenu;
    }

    bool hasSubMenu() {
       return subMenu != NULL;
    }

    bool isSelected() const {
        return bItemSelected;
    }

    void setSelected(bool isItemSelected);

        private: //!< callback Slots (private overriders)

    void doMouseUnclick(const int btnId);

        public:  //!< Signals

    struct {
        /**
         * Emit callback Slot on menu item click.
         * @param theUserData (const size_t ) - user predefined data.
         */
        StSignal<void (const size_t )> onItemClick;
    } signals;

};

#endif //__StGLMenuItem_h_
