/**
 * Copyright © 2009-2010 Kirill Gavrilov <kirill@sview.ru>
 *
 * Distributed under the Boost Software License, Version 1.0.
 * See accompanying file license-boost.txt or copy at
 * http://www.boost.org/LICENSE_1_0.txt
 */

#ifndef __StGLMessageBox_h_
#define __StGLMessageBox_h_

#include <StGL/StGL.h>
#include <StGLWidgets/StGLTextArea.h>
#include <StGLWidgets/StGLMenuProgram.h>

class ST_LOCAL StGLMessageBox : public StGLWidget {

        private:

    StGLTextArea*     stTextArea;
    StGLMenuProgram    stProgram;
    StGLVertexBuffer stVertexBuf;

        public:

    StGLMessageBox(StGLWidget* parent, const StString& text,
                   const int& width = 384, const int& height = 128);
    virtual ~StGLMessageBox();
    virtual bool stglInit();
    virtual void stglResize();
    virtual void stglResize(const StRectI_t& winRectPx);
    virtual void stglDraw(unsigned int view);
    virtual void setVisibility(bool isVisible, bool isForce);
    virtual bool tryClick(const StPointD_t& cursorZo, const int& mouseBtn, bool& isItemClicked);
    virtual bool tryUnClick(const StPointD_t& cursorZo, const int& mouseBtn, bool& isItemUnclicked);

        private:   //!< callback Slots (private overriders)

    void doMouseUnclick(const int btnId);

        public:    //!< Signals

    struct {
        /**
         * @param theUserData (const size_t ) - user predefined data.
         */
        StSignal<void (const size_t )> onClickLeft;
        StSignal<void (const size_t )> onClickRight;
    } signals;

        public:    //!< callback Slots

    void doKillSelf(const size_t );

};

#endif //__StGLMessageBox_h_
