/**
 * Copyright © 2009-2011 Kirill Gavrilov <kirill@sview.ru>
 *
 * Distributed under the Boost Software License, Version 1.0.
 * See accompanying file license-boost.txt or copy at
 * http://www.boost.org/LICENSE_1_0.txt
 */

#ifndef __StGLTextArea_h_
#define __StGLTextArea_h_

#include <StTemplates/StArrayList.h>

#include <FTGL/ftgl.h>
#include <StGL/StGLVertexBuffer.h>
#include <StGL/StGLVec.h>
#include <StGLWidgets/StGLShare.h>
#include <StGLWidgets/StGLWidget.h>

///TODO (Kirill Gavrilov#9) implement (partial) HTML tags support
/**enum TextFormat {
    ST_TEXT_PLAIN = 0,
    ST_TEXT_HTML,
};*/

/**
 * Class implements basic text rendering widget.
 */
class ST_LOCAL StGLTextArea : public StGLWidget {

        public:

    typedef enum {
        SIZE_NORMAL,
        SIZE_BIG,
        SIZE_SMALL,
    } FontSize;

    enum {
        TO_RIGHT = 1 << 0,
        TO_LEFT  = 1 << 1,
        TO_DOWN  = 1 << 2,
        TO_UP    = 1 << 3,
    };

    enum {
        ALIGN_LEFT    = FTGL::ALIGN_LEFT,
        ALIGN_CENTER  = FTGL::ALIGN_CENTER,
        ALIGN_RIGHT   = FTGL::ALIGN_RIGHT,
        ALIGN_JUSTIFY = FTGL::ALIGN_JUSTIFY,
    };

        private:

    StArrayList<StGLVertexBuffer> stTextVertBuf;
    StArrayList<StGLVertexBuffer> stTextTCrdBuf;

    StGLVertexBuffer stBorderIVertBuf;
    StGLVertexBuffer stBorderOVertBuf;

    class StTextProgram;
    class StBorderProgram;
    StGLShare<StTextProgram> stTextProgram;
    StGLShare<StBorderProgram> stBorderProgram;

        private:

    void drawText();

        protected:

    StGLShare<FTTextureFont> ftFont;
    FTSimpleLayout fontLayout;
    FTBBox              ftBox;
    StString             text;
    int         areaDirection;
    FontSize           mySize;
    StGLVec4      myTextColor;
    StGLVec4    myShadowColor;
    StGLVec4        backColor;
    StGLVec4      borderColor;

    StPointI_t      textPoint;

    bool      toRecomputeText;
    bool            isCutView;
    bool     isShowAreaBorder;
    bool       myToDrawShadow;
    bool            myToScale;
    bool        isInitialized;

    void recomputeBorder();

        public:

    StGLTextArea(StGLWidget* theParent,
                 const int theLeft = 32, const int theTop = 32,
                 const StGLCorner theCorner = StGLCorner(ST_VCORNER_TOP, ST_HCORNER_LEFT),
                 const int theWidth = 256, const int theHeight = 32, bool isCutView = false,
                 const FontSize theSize = StGLTextArea::SIZE_NORMAL,
                 const bool toDrawShadow = false,
                 const bool toScale = false);

    virtual ~StGLTextArea();

    virtual const StString& getClassName();

    const StString& getText() const;
    void setText(const StString& text);

    void setBorder(bool toShow) {
        isShowAreaBorder = toShow;
    }

    void setBorderColor(const GLfloat& r, const GLfloat& g, const GLfloat& b) {
        borderColor.r() = r;
        borderColor.g() = g;
        borderColor.b() = b;
    }

    void setBackColor(const GLfloat& r, const GLfloat& g, const GLfloat& b) {
        backColor.r() = r;
        backColor.g() = g;
        backColor.b() = b;
    }

    void setTextColor(const GLfloat& r, const GLfloat& g, const GLfloat& b) {
        myTextColor.r() = r;
        myTextColor.g() = g;
        myTextColor.b() = b;
    }

    void setAreaDirection(const int& direction) {
        areaDirection = direction;
    }

    void setTextAligment(const int& aligment);
    void setTextWidth(const int theWidth);

    const int getFontSize() const {
        switch(mySize) {
            case SIZE_SMALL:
                return 12;
            case SIZE_BIG:
                return 28;
            case SIZE_NORMAL:
            default:
                return 16;
        }
    }

    virtual bool stglInit();
    virtual void stglDraw(unsigned int view);

};

#endif //__StGLTextArea_h_
