/**
 * Copyright © 2009-2010 Kirill Gavrilov <kirill@sview.ru>
 *
 * Distributed under the Boost Software License, Version 1.0.
 * See accompanying file license-boost.txt or copy at
 * http://www.boost.org/LICENSE_1_0.txt
 */

#ifndef __StFPSMeter_h_
#define __StFPSMeter_h_

#include "StTimer.h"

/**
 * Simple class for FPS measurements, based on StTimer.
 */
class ST_LOCAL StFPSMeter {

        private:

    StTimer     stTimer; // main timer
    double  framesCount; // frames counter
    double  measureStep; // time interval to compute average FPS (in seconds)
    double   averageFPS; // last computed average FPS
    bool       bUpdated;

        public:

    StFPSMeter()
    : stTimer(true),
      framesCount(0.0),
      measureStep(1.0),
      averageFPS(0.0),
      bUpdated(true) {
        //
    }

    virtual ~StFPSMeter() {
        //
    }

    bool isUpdated() {
        if(bUpdated) {
            bUpdated = false;
            return true;
        } else {
            return false;
        }
    }

    /**
     * Increment frames counter.
     */
    virtual bool nextFrame() {
        framesCount += 1.0;
        double elapsedSec = stTimer.getElapsedTimeInSec();
        if(elapsedSec >= measureStep) {
            bUpdated = true;
            averageFPS = framesCount / elapsedSec;

            // restart the timer for next measurements
            stTimer.restart(); framesCount = 0.0;
            return true;
        }
        return false;
    }

    /**
     * Increment frames counter.
     */
    StFPSMeter& operator++() {
        nextFrame();
        return (*this);
    }

    /**
     * @return fps (double ) - average FPS.
     */
    double getAverage() const {
        return averageFPS;
    }

    /**
     * @return delay (double ) - average delay.
     */
    double getDelayMs() const {
        return 1000.0 / averageFPS;
    }

};

#endif // __StFPSMeter_h_
